from django.shortcuts import render
# from mainApp.models import *
from clhrApp.models import *
from clhrApp.serializers import *
from django.shortcuts import render,redirect
from django.contrib.auth.decorators import login_required,user_passes_test
from django.utils.decorators import method_decorator
from clmainApp.utils import *
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from datetime import datetime,timedelta
from django.db.models import Q
from django.db.models.functions import Coalesce
from django.db.models import Count,Max,Sum
from brmainApp.models import DrData

######### التخصصات #########
class DepartmentsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_hrDepartments'), login_url='/auth/login/'))
    def get(self,request):
        #         branchId = getBranchId(request)
        #if not branchId : return redirect('clauthApp:home')
        all_departments =Department.objects.all()
        all_departments_serialized=DepartmentSerializer(all_departments,many=True)
        return render(request,"Clinic/clhrApp/departments.html",{"all_departments":all_departments_serialized.data,'activePage':'Departments'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_hrDepartment'), login_url='/auth/login/'))    
    def post(self,request):
        #         branchId = getBranchId(request)
        #if not branchId : return redirect('clauthApp:home')
        serializedObj= DepartmentSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class DepartmentApi(APIView):
    def get_object(self,pk):
        try:
            return Department.objects.get(pk=pk)
        except Department.DoesNotExist:
            return None    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_hrDepartments'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializeobj= DepartmentSerializer(Obj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_hrDepartment'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= DepartmentSerializer(obj,data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_hrDepartment'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

##### Employees  الموظفين########
class EmployeesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_Employees'), login_url='/main/login/'))
    def get(self,request):
        # branchId = getBranchId(request)
        # if not branchId : return redirect('clauthApp:home')
        departments =Department.objects.all()
        branches = Branch.objects.all()
        # departments =Department.objects.filter(is_clinic=False)
        all_emps =Employee.objects.all().order_by("-is_active")
        all_emps_srl=EmployeeSerializer(all_emps,many=True)
        return render(request,"Clinic/clhrApp/employees.html",{"all_emps":all_emps_srl.data,"departments":departments,"branches":branches,
        'activePage':'Employees'})

    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_Employee'), login_url='/main/login/'))    
    @method_decorator(user_passes_test(lambda u: any(u.has_perm(perm) for perm in ['BrManagement.add_BranchEmployee', 'ClManagement.add_Employee']), login_url='/auth/login/'))
    def post(self,request):
        # branchId = getBranchId(request)
        # if not branchId : return redirect('clauthApp:home')
        request.data._mutable = True
        created_by_id = request.user.id
        request.data.update({"created_by":created_by_id})
        serializedObj= EmployeeSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            images = request.FILES.getlist("ea_files")
            if len(images) > 1:
                image_objs = []
                counter=1
                for img in images:
                    img_obj = {
                        "employee":serializedObj.data["id"],
                        "ea_label":" ملف {}".format(counter),
                        "ea_file":img,
                        "created_by":created_by_id,
                    }
                    image_objs.append(img_obj)
                    counter=counter+1
                attaches = EmpAttachmentSerializer(data=image_objs,many=True)
                if attaches.is_valid():
                    attaches.save()
                else :return Response(attaches.errors,status=status.HTTP_400_BAD_REQUEST)
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EmployeeApi(APIView):
    def get_object(self,pk):
        try:
            return Employee.objects.get(pk=pk)
        except Employee.DoesNotExist:
            return None    
    
    # @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_patients'), login_url='/main/login/'))        
    # def get(self,request,pk): 
    #     empObj=self.get_object(pk)
    #     if empObj :
    #         context = {"emp":empObj,'activePage':'Employees'}
    #         all_earnings = Earning.objects.all()
    #         all_deductions = Deduction.objects.all()
    #         context.update({"all_earnings":all_earnings,"all_deductions": all_deductions})
    #         empSalaries = EmpSalary.objects.filter(emp_id=pk).order_by('-to_day')
    #         not_paid_salary = empSalaries.filter(pay_day__isnull=True).count()
    #         max_pay_day = empSalaries.aggregate(Max('pay_day'))['pay_day__max']
    #         max_pay_dayObjs = empSalaries.filter(pay_day = max_pay_day)
    #         if max_pay_dayObjs : 
    #             last_to_day = max_pay_dayObjs[0].to_day
    #             day_after_last_to_day = last_to_day+timedelta(days=1)
    #         else : day_after_last_to_day = None
    #         context.update({"all_emp_salaries":empSalaries ,
    #          'day_after_last_to_day':day_after_last_to_day,
    #          'emp_payday_max':empSalaries.aggregate(Max('pay_day'))['pay_day__max'],
    #          "not_paid_salary":not_paid_salary})
    #         empEarnings = EmpEarning.objects.filter(emp_id=pk ,is_done=False)
    #         context.update({"all_emp_earnings":empEarnings , 'emp_earnings_sum':empEarnings.aggregate(Sum('amount'))['amount__sum']})

            
    #         empDeduction = EmpDeduction.objects.filter(emp_id=pk, is_done=False)
    #         context.update({"all_emp_deductions":empDeduction , 'emp_deductions_sum':empDeduction.aggregate(Sum('amount'))['amount__sum']})
    #         # patientVisitObjs = PatientVisit.objects.filter(patient=patientObj,visit_status__in=["1I","4F"]).order_by('-visit_day')
    #         # patientVisitObjs_serialized = PatientVisitMiniSerializer(patientVisitObjs,many=True)
    #         # context.update({'patientVisitObjs':patientVisitObjs_serialized.data})
            
    #         # medical_services =MedicalService.objects.all().values('id', 'name','price')
    #         # context.update({"medical_services":medical_services,})           
    #         # blood_type_choices = ChoiceField(choices=BLOOD_TYPE_CHOICES)
    #         # context.update({"blood_types":blood_type_choices.choices,})
    #         # context.update({"pageTitle":patientObj.fullname,})
            
    #         return render(request,"clhrApp/Managment/employeePages/employee.html",context)
    #     else : return redirect('/main/login/')

    @method_decorator(user_passes_test(lambda u: any(u.has_perm(perm) for perm in ['BrManagement.change_BranchEmployee', 'ClManagement.change_Employee']), login_url='/auth/login/'))
    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_Employee'), login_url='/main/login/'))
    def put(self,request,pk):
        request.data._mutable = True
        request.data.update({"created_by":request.user.id})
        obj=self.get_object(pk)
        serializedObj= EmployeeSerializer(obj,data=request.data,partial=True)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: any(u.has_perm(perm) for perm in ['BrManagement.delete_BranchEmployee', 'ClManagement.delete_Employee']), login_url='/auth/login/'))
    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_Employee'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class EmpEvaluationsApi(APIView):
    def get_object(self,emp_id):
        try:
            return Employee.objects.get(pk=emp_id)
        except Employee.DoesNotExist:
            return None
    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_Employee'), login_url='/main/login/'))
    def get(self,request,emp_id):
        # branchId = getBranchId(request)
        # if not branchId : return redirect('clauthApp:home')
        emp_obj = self.get_object(emp_id)
        if emp_obj :
            emp_evaluations =EmpEvaluation.objects.filter(employee_id = emp_id)
            #all_emps_srl=EmpEvaluationSerializer(all_emps,many=True)
            return render(request,"Clinic/clhrApp/emp_evaluations.html",{
                "emp_evaluations":emp_evaluations,
                "emp_obj":emp_obj,
                "emp_id" : emp_id,
                'activePage':'Employees'})
        else : return redirect('clhrApp:EmployeesApi')

    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_Employee'), login_url='/main/login/'))    
    def post(self,request,emp_id):
        # branchId = getBranchId(request)
        # if not branchId : return redirect('clauthApp:home')
        data = request.data
        data._mutable = True
        created_by_id = request.user.id
        data.update({"created_by":created_by_id,'employee':emp_id})
        serializedObj= EmpEvaluationSerializer(data=data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EmpEvaluationApi(APIView):
    def get_object(self,emp_evaluation_id):
        try:
            return EmpEvaluation.objects.get(pk=emp_evaluation_id)
        except EmpEvaluation.DoesNotExist:
            return None

    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.edit_Employee'), login_url='/main/login/'))
    def put(self,request,emp_evaluation_id):
        data =  request.data
        data._mutable = True
        #data.update({"created_by":request.user.id})
        obj=self.get_object(emp_evaluation_id)
        serializedObj= EmpEvaluationSerializer(obj,data=data,partial=True)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_Employee'), login_url='/main/login/'))
    def delete(self,request,emp_evaluation_id):
        obj=self.get_object(emp_evaluation_id)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

##### Deductions   بنود الخصومات ########
class DeductionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_deductions'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_deductions =Deduction.objects.all()
        all_deductions_serialized=DeductionSerializer(all_deductions,many=True)
        return render(request,"Clinic/clhrApp/deductions.html",{"all_deductions":all_deductions_serialized.data,
        'activePage':'Deductions'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_deductions'), login_url='/main/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        serializedObj= DeductionSerializer(data=request.data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class DeductionApi(APIView):
    def get_object(self,pk):
        try:
            return Deduction.objects.get(pk=pk)
        except Deduction.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_deductions'), login_url='/main/login/'))        
    def get(self,request,pk): 
        deductionObj=self.get_object(pk)
        if deductionObj :
            serializeobj= DeductionSerializer(deductionObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_deductions'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= DeductionSerializer(obj,data=request.data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_deductions'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

##### Earnings  الاستحقاقات والمكافأت بنود ########
class EarningsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_earnings'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_earnings =Earning.objects.all()
        all_earnings_serialized=EarningSerializer(all_earnings,many=True)
        return render(request,"Clinic/clhrApp/earnings.html",{"all_earnings":all_earnings_serialized.data,
        'activePage':'Earnings'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_earnings'), login_url='/main/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        serializedObj= EarningSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EarningApi(APIView):
    def get_object(self,pk):
        try:
            return Earning.objects.get(pk=pk)
        except Earning.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_earnings'), login_url='/main/login/'))        
    def get(self,request,pk): 
        earningObj=self.get_object(pk)
        if earningObj :
            serializeobj= EarningSerializer(earningObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_earnings'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= EarningSerializer(obj,data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_earnings'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

from brmainApp.models import DrDeduction,DrEarning
from brsessionApp.models import PatientSessionItem
import operator
from functools import reduce
from brsessionApp.serializers import PsiAssessment

class EmpSalariesReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_emp_dr_salaries'), login_url='/main/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values("id","name")
        #session_status = SessionStatus.objects.all().order_by("code")
        return render(request,"Clinic/clhrApp/emp_salaries_report.html",{
            #"locations_names":locations_names,"locations_count":locations_count,
            "branches":branches,
            'activePage':'EmpSalariesReport'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_emp_dr_salaries'), login_url='/main/login/'))
    def post(self,request):
        data=request.data
        data._mutable=True
        startDate = data["startDate"]
        endDate = data["endDate"]
        branch_id = data["branch_id"]
        employess_doctors_data =[]
        dr_levels = DrLevel.objects.filter(branch_id=branch_id)
        branch_doctors = DrData.objects.filter(branch_id=branch_id,is_active=True).values('id','dr_id','dr__name','month_salary','salary_phone_number',"dr_level_id")
        branch_doctors_ids = branch_doctors.values_list('id',flat=True)#dr data objects 
        branch__doctors_ids = branch_doctors.values_list('dr_id',flat=True)#dr_id  object
        #import pdb
        #pdb.set_trace()
        #.values("id","confirmed_by_id","doctor_id","session_status_id","session__patient_id",'assessment_id','assessment_s')
        ps_items_all = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,doctor_id__in=branch__doctors_ids,session_day__range=[startDate,endDate]).values("id","session_status_id","session__patient_id","assessment_id","assessment_s_id")

        #ps_items = ps_items_all.filter(reduce(operator.or_, [Q(session_status_id="2AT"),Q(psi_assessment_choice__in=["B","D"]),Q(psi_assessment_choice__in=["B","D"])]))
        # جميع الجلسات خلال الفترة اللى متاخدش عليها اجراء 
        psi_sessions_without_B_and_D__data = ps_items_all.filter(psi_assessment_choice="UD",psi_assessment_choice_s="UD")
        psi_sessions_without_B_and_D_list_ids = psi_sessions_without_B_and_D__data.values_list('id',flat=True)
        #جميع الجلسات خلال الفترة  خدو عليها بونص او خصم  
        psi_sessions_with_B_and_D__data = ps_items_all.exclude(id__in=psi_sessions_without_B_and_D_list_ids)

        psi_assessments_ids_all = psi_sessions_with_B_and_D__data.values('assessment_id','assessment_s')
        psi_assessments_ids = []
        for assessment_obj in psi_assessments_ids_all :
            if not assessment_obj["assessment_id"] and not assessment_obj["assessment_s"] : continue
            else :
                if assessment_obj["assessment_id"] : psi_assessments_ids.append(assessment_obj["assessment_id"])
                if assessment_obj["assessment_s"] : psi_assessments_ids.append(assessment_obj["assessment_s"])
        
        psi_assessments_ids_without_None = set(psi_assessments_ids)#still contain None , next line will remove NOne 
        psi_assessments_ids_without_None.discard(None)
        psi_assessments = PsiAssessment.objects.filter(id__in=psi_assessments_ids_without_None).values('id','doctor','assessment_choice','sessions_count')

        branch_doctors_earnings = DrEarning.objects.filter(dr_data_id__in=branch_doctors_ids,created_at__date__range=[startDate,endDate] ,is_done=False).values('id','amount').order_by('-created_at')
        branch_doctors_earnings_ids = branch_doctors_earnings.values_list('id',flat=True)
        branch_doctors_deductions = DrDeduction.objects.filter(dr_data_id__in=branch_doctors_ids,created_at__date__range=[startDate,endDate] ,is_done=False).values('id','amount').order_by('-created_at')
        branch_doctors_deductions_ids = branch_doctors_deductions.values_list('id',flat=True)
        #import pdb
        #pdb.set_trace()
        ps_items_attend = ps_items_all.filter(session_status_id="2AT")
        for dr_data_obj in branch_doctors:
            #dr_data_obj = branch_doctors.filter(id=emp_id)[0]
            dr_data_id = dr_data_obj["id"]
            dr_id = dr_data_obj["dr_id"]
            dr_level_obj = dr_levels.get(id=dr_data_obj["dr_level_id"])
            dr_ps_items_attend = ps_items_attend.filter(doctor_id=dr_id)
            #sessions_count and amount 
            ps_items_attend_count  = dr_ps_items_attend.count()
            ps_items_attend_amount = ps_items_attend_count * dr_level_obj.session_price
            # all dr_assessments
            dr_assessments = psi_assessments.filter(doctor_id=dr_id)
            #bouns count and amount 
            dr_Bonus_list = dr_assessments.filter(assessment_choice="B")
            dr_Bonus_list_ids = dr_Bonus_list.values_list('id',flat=True)
            dr_ps_items_bouns_count = ps_items_all.filter(Q(assessment_id__in=dr_Bonus_list_ids)|Q(assessment_s_id__in=dr_Bonus_list_ids)).count()
            #ps_items_bouns_count  = ps_items.filter(reduce(operator.or_, [Q(psi_assessment_choice="B"),Q(psi_assessment_choice="B")]),reduce(operator.or_, [Q(psi_assessment_choice="B"),Q(psi_assessment_choice="B")])).count()
            ps_items_bouns_amount = dr_ps_items_bouns_count * dr_level_obj.bonus_amount
            # deduction and amount 
            dr_Deduction_list = dr_assessments.filter(assessment_choice="D")
            dr_Deduction_list_ids = dr_Deduction_list.values_list('id',flat=True)
            dr_ps_items_deduction_count = ps_items_all.filter(Q(assessment_id__in=dr_Deduction_list_ids)|Q(assessment_s_id__in=dr_Deduction_list_ids)).count()            
            ps_items_deduction_amount = dr_ps_items_deduction_count * dr_level_obj.deduction_amount

            dr_data_obj ={"emp_id":dr_data_id,"emp_name":dr_data_obj["dr__name"],"month_salary":dr_data_obj["month_salary"],"number_1":dr_data_obj["salary_phone_number"]}
            #dr_sessions = ps_items.filter(confirmed_by_id=dr_id)
            doctor_earnings = branch_doctors_earnings.filter(dr_data_id=dr_data_id)
            dr_data_obj["emp_earnings_list"] = doctor_earnings.values_list('id',flat=True)
            dr_data_obj["emp_sum_earnings"] = doctor_earnings.values('amount').aggregate(sum_earnings=Coalesce(Sum('amount'),0))['sum_earnings']
            dr_data_obj["emp_count_earnings"] = doctor_earnings.values('id').aggregate(count_earnings=Coalesce(Count('id'),0))['count_earnings']

            doctor_deductions = branch_doctors_deductions.filter(dr_data_id=dr_data_id)
            dr_data_obj["emp_deductions_list"] = doctor_deductions.values_list('id',flat=True)
            dr_data_obj["emp_sum_deductions"] = doctor_deductions.values('amount').aggregate(sum_deductions=Coalesce(Sum('amount'),0))['sum_deductions']
            dr_data_obj["emp_count_deductions"] = doctor_deductions.values('id').aggregate(count_deductions=Coalesce(Count('id'),0))['count_deductions']
            dr_data_obj.update({"ssn_Attended_count":ps_items_attend_count,"ssn_Bonus_count":dr_ps_items_bouns_count,"ssn_Deduction_count":dr_ps_items_deduction_count})
            dr_data_obj.update({"ssn_Attended":ps_items_attend_amount,"ssn_Bonus":ps_items_bouns_amount,"ssn_Deduction":ps_items_deduction_amount,"dr_level_name":dr_level_obj.name})

            sessions_net_amount = (ps_items_attend_amount + ps_items_bouns_amount ) - ps_items_deduction_amount
            dr_net_amount_as_emp = (dr_data_obj["month_salary"] + dr_data_obj["emp_sum_earnings"] ) - dr_data_obj["emp_sum_deductions"]
            dr_data_obj["emp_salary"] = sessions_net_amount + dr_net_amount_as_emp

            employess_doctors_data.append(dr_data_obj)

        branch_employees = Employee.objects.filter(branch_id=branch_id,is_active=True).values('id','fullname','month_salary','number_1')
        branch_employees_ids = branch_employees.values_list('id',flat=True)
        branch_employees_earnings = EmpEarning.objects.filter(emp_id__in=branch_employees_ids,created_at__date__range=[startDate,endDate] ,is_done=False).values('id','amount').order_by('-created_at')
        branch_employees_earnings_ids = branch_employees_earnings.values_list('id',flat=True)
        branch_employees_deductions = EmpDeduction.objects.filter(emp_id__in=branch_employees_ids,created_at__date__range=[startDate,endDate] ,is_done=False).values('id','amount').order_by('-created_at')
        branch_employees_deductions_ids = branch_employees_deductions.values_list('id',flat=True)
        for emp_data in branch_employees:
            #emp_data = branch_employees.filter(id=emp_id)[0]
            emp_id = emp_data["id"]
            emp_obj ={"emp_id":emp_id,"emp_name":emp_data["fullname"],"month_salary":emp_data["month_salary"],"number_1":emp_data["number_1"]}
            #dr_sessions = ps_items.filter(confirmed_by_id=dr_id)
            employee_earnings = branch_employees_earnings.filter(emp_id=emp_id)
            emp_obj["emp_earnings_list"] = employee_earnings.values_list('id',flat=True)
            emp_obj["emp_sum_earnings"] = employee_earnings.values('amount').aggregate(sum_earnings=Coalesce(Sum('amount'),0))['sum_earnings']
            emp_obj["emp_count_earnings"] = employee_earnings.values('id').aggregate(count_earnings=Coalesce(Count('id'),0))['count_earnings']

            employee_deductions = branch_employees_deductions.filter(emp_id=emp_id)
            emp_obj["emp_deductions_list"] = employee_deductions.values_list('id',flat=True)
            emp_obj["emp_sum_deductions"] = employee_deductions.values('amount').aggregate(sum_deductions=Coalesce(Sum('amount'),0))['sum_deductions']
            emp_obj["emp_count_deductions"] = employee_deductions.values('id').aggregate(count_deductions=Coalesce(Count('id'),0))['count_deductions']
            emp_obj["emp_salary"] = (emp_obj["month_salary"] + emp_obj["emp_sum_earnings"]) - emp_obj["emp_sum_deductions"]
            emp_obj.update({"ssn_Attended_count":0,"ssn_Bonus_count":0,"ssn_Deduction_count":0})
            emp_obj.update({"ssn_Attended":"","ssn_Bonus":"","ssn_Deduction":"","dr_level_name":""})
            employess_doctors_data.append(emp_obj)

        
        response_data = {"employess_doctors_data":employess_doctors_data,
                         "emp_earnings_data":branch_employees_earnings_ids,"dr_earnings_data":branch_doctors_earnings_ids,
                         "emp_deductions_data":branch_employees_deductions_ids,"dr_earnings_data":branch_doctors_deductions_ids,
                         }
        return Response(response_data , status=status.HTTP_201_CREATED)

class DrSalariesReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_dr_salaries'), login_url='/main/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values("id","name")
        #session_status = SessionStatus.objects.all().order_by("code")
        return render(request,"Clinic/clhrApp/dr_salaries_report.html",{
            #"locations_names":locations_names,"locations_count":locations_count,
            "branches":branches,
            'activePage':'DrSalariesReport'})


class EmpEarningsDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        emp_earnings_ids = data.getlist('salary_details[]')
        if '' in  emp_earnings_ids : emp_earnings_ids.pop(emp_earnings_ids.index(''))
        emp_earnings = EmpEarning.objects.filter(id__in=emp_earnings_ids)
        emp_earnings_data = EmpEarningSerializer(emp_earnings,many=True).data
        return Response(emp_earnings_data , status=status.HTTP_200_OK)

class EmpDeductionsDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        emp_deductions_ids = data.getlist('salary_details[]')
        if '' in  emp_deductions_ids : emp_deductions_ids.pop(emp_deductions_ids.index(''))
        emp_deductions = EmpDeduction.objects.filter(id__in=emp_deductions_ids)
        emp_deductions_data = EmpDeductionSerializer(emp_deductions,many=True).data
        return Response(emp_deductions_data , status=status.HTTP_200_OK)
    
# class EmpActionApi(APIView):
#     @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_EmpAction'), login_url='/main/login/'))
#     def get(self,request):
#                 branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
#         emps =Employee.objects.filter(is_active=True)
#         emps_srl=EmployeeSerializer(emps,many=True)
#         ern =Earning.objects.all()
#         ern_srl=EarningSerializer(ern,many=True)
#         ds =Deduction.objects.all()
#         ds_srl=DeductionSerializer(ds,many=True)
#         return render(request,"clhrApp/Managment/emp_action.html",{
#             "emps":emps_srl.data,
#             "ern":ern_srl.data,
#             "ds" :ds_srl.data,
#             'activePage':'EmpAction'})

############## END Employee%Module #################

# ###### {Pres}   ########
# class PresApi(APIView):
#     def get(self,request):
#         return render(request,"clhrApp/Managment/pres2.html",)

# ###### {CompanyRptApi}   ########
# class CompanyRptApi(APIView):
#     def get(self,request):
#         all_PatientVisit = PatientVisit.objects.all()
#         return render(request,"clhrApp/Managment/company_rpt.html",{"all_PatientVisit":all_PatientVisit})



# # ###### DrugUsages    الجرعات########


############## HR%Reception%Module #################

class EmployeesAttendanceApi(APIView):
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        departments=  Department.objects.filter(d_category__in=["1G","3S","4R"]).order_by("d_order")
        att = EmpAttendance.objects.filter(check_out_by__isnull=True).order_by('emp__department__d_order')
        inside_employees_ids = att.values_list('emp_id',flat=True)
        serializer = EmpAttendanceCustomSerializer(att,many=True)
        all_employees = Employee.objects.filter(~Q(id__in=inside_employees_ids)).values("id","fullname","department_id")
        return render(request,"clhrApp/Reception/attendancePages/emp_attendance.html",{
        "attendaces":serializer.data,
        "all_employees":all_employees,
        "departments":departments,
        "pageTitle":"الموظفين حضور وإنصراف",
        })
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        request.data._mutable = True
        request.data.update({"check_in_by":request.user.id})
        serializedObj= EmpAttendanceCustomSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EmployeeAttendanceApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.edit_Earning'), login_url='/main/login/'))
    def put(self,request,pk):
        if request.data : request.data._mutable = True
        request.data.update({"check_out":timezone.now()})
        request.data.update({"check_out_by":request.user.id})
        obj=EmpAttendance.objects.get(pk=pk)
        serializedObj= EmpAttendanceSerializer(obj,data=request.data,partial=True)
        if serializedObj.is_valid():
            serializedObj.save()
            employee = Employee.objects.filter(id=obj.emp_id).values("id","fullname","department_id").first()
            return Response(employee , status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)


############## END HR%Reception%Module #################





